/********************************
 *
 * compare the reference ligand with the new position
 * 
 *
 *
 * Authors: Sébastien Verel, Sébastien Fiorucci
 * 
 * Date:
 *   version 1 - 12/11/2013
 *
 *******************************/

// c++ in-out 
#include <iostream>
// random numbers 
#include <random>

// include to have solution representation
#include <minimizers/solution/solution.h>

// includes to compute the energy function
#include <attractrigidbody.h>
#include <minimizers/solution/eval.h>

// initialization
#include <minimizers/init/fixedDistanceInit.h>

// to save
#include <pdbio.h>

#include <rmsd.h>

/*
 * The main function
 *
 * Arguments:
 *   1 : receptor file (required)
 *   2 : reference ligand file   (required)
 *   3 : random ligand file   (required)
 *   4 : solution file   (required)
 *
 * Out:
 *  Energy reference ligand
 *  Energy new ligand
 *  RMSD between ligands
 */
int main(int argc, char** argv) {

  //-----------------------------------------------
  // Fitness function (energy function)

  // read the receptor and the ligand
  PTools::AttractRigidbody receptor(argv[1]);
  PTools::AttractRigidbody ligandRef(argv[2]);
  PTools::AttractRigidbody ligandRnd(argv[3]);

  // constructor with: parameters of the field, file with the receptor, file with the ligand
  Eval evalref("../data/parmw.par", receptor, ligandRef);

  Eval eval("../data/parmw.par", receptor, ligandRnd);
  
  //-----------------------------------------------
  // Energy of the reference

  // the solution
  Solution solution;
  
  // compute the energy of the solution
  evalref.apply(solution);

  // print the solution on the output
  //  solution.printOn();
  //  std::cout << std::endl;
  std::cout << solution.fitness() << " ";

 
  //-----------------------------------------------
  // Energy of the solution

  std::fstream file(argv[4]);

  if (file) {
    solution.readFrom(file);

    //    solution.printOn();
    //    std::cout << std::endl;

    // compute the energy of the solution
    eval.apply(solution);
    std::cout << solution.fitness() << " ";
    
    // deviation from the reference
    solution.move(ligandRnd);
    dbl r = PTools::Rmsd(ligandRef, ligandRnd);
    std::cout << r << std::endl;

    file.close();
  } else {
    std::cerr << "Error file not found: " << argv[4] << std::endl;
  }
}
