/********************************
 *
 * Example of evolution strategy : random search
 * based on Ptools
 *
 *
 * Authors: 
 * 
 * Date:
 *   version 2 - 12/11/2014
 *   version 1 - 05/11/2013
 *
 *******************************/

// in-out of c++
#include <iostream>
// random numbers 
#include <random>

// include to have solution representation
#include <minimizers/solution/solution.h>

// includes to compute the energy function
#include <attractrigidbody.h>
#include <minimizers/solution/eval.h>

// initialization
#include <minimizers/init/fixedDistanceInit.h>

// example of minimizer class
#include <minimizers/algo/randomES.h>

/*
 * The main function
 */
int main(int argc, char** argv) {
  // random generator
  std::mt19937 rng;
  
  if (argc > 3)
    // the random seed is given by the user to be able to re-run the experiment
    rng.seed(atoi(argv[3]));
  else
    // the random seed is given by time
    rng.seed(time(0));

  //-----------------------------------------------
  // Fitness function (energy function)

  // read the receptor and the ligand
  PTools::AttractRigidbody receptor(argv[1]);
  PTools::AttractRigidbody ligand(argv[2]);

  // constructor with: parameters of the field, file with the receptor, file with the ligand
  Eval eval("../data/parmw.par", receptor, ligand);
  
  //-----------------------------------------------
  // Example of minimization

  // initialization method (see example2_init.cpp)
  FixedDistanceInit init(rng, receptor, ligand);
  
  // the random search
  RandomES minimizer(rng, eval, init);

  minimizer.run();

  // the final (best) solution
  Solution solution = minimizer.finalSolution();
  
  // print the solution on the output
  solution.printOn();
  std::cout << std::endl;

}
