#ifndef __ES__h
#define __ES__h

/*
 * Description:
 *   Abstract class for Evolution Strategy
 *
 * Author : SV
 *
 * Date   : 
 *   version 2 - 2014-11-12
 *   version 1 - 2013-11-??
 */

#include <minimizers/solution/solution.h>
#include <minimizers/solution/eval.h>

class EvolutionStrategy {
public:
  /*
   * Empty constructor
   * 
   * @param _rng random generator
   * @param _eval evaluation function
   */
  EvolutionStrategy(std::mt19937 & _rng, Eval & _eval) : rng(_rng), eval(_eval) {
  }

  /*
   * Execute the optimization algorithm
   * (pure virtual method)
   */
  virtual void run() = 0 ;

  /*
   * give the best solution
   */
  Solution finalSolution() {
    return bestSolution;
  }

protected:
  // random generator
  std::mt19937 & rng;

  // evaluation function
  Eval & eval;

  // the best solution at the end of the algorithm
  Solution bestSolution;
};

#endif
