#ifndef __Solution__h
#define __Solution__h

/*
 * Solution representation
 *
 * Author : SV
 * Date   :
 *   version 1 - 2013-11-05
 */

#include <basetypes.h>
#include <src/attractrigidbody.h>

class Solution {
public:
  // the 6 coordinates of the solution
  Vdouble coord ;
  
  /*
   * Empty constructor
   */
  Solution() {
    coord.resize(6);
    for(int i = 0; i < 6; i++)
      coord[i] = 0;
    
    // default value
    energy = 10000;
  }
  
  /*
   * Copy constructor
   */
  Solution(const Solution & _sol) {
    if (_sol.coord.size() > 0) {
      coord.resize(6);
      for(int i = 0; i < 6; i++)
        coord[i] = _sol.coord[i];
    }
    fitness(_sol.fitness());
  }
  
  /*
   * Get the fitness
   */
  dbl fitness() const {
    return energy;
  }
  
  /*
   * Set the fitness
   */
  void fitness(dbl _energy) {
    energy = _energy;
  }
  
  /*
   * Print the solution
   */
  void printOn() {
    std::cout << fitness() ;
    
    for(int i = 0; i < 6; i++)
      std::cout << " " << coord[i] ;
  }
  
  /*
   * Print the solution on the output stream that could be a file
   */
  void printOn(std::ostream & _os) {
    _os << fitness() ;
    
    for(int i = 0; i < 6; i++)
      _os << " " << coord[i] ;
  }
  
  /*
   * Print the solution on the output stream that could be a file
   */
  void readFrom(std::istream & _is) {
    double fit;
    _is >> fit;
    fitness(fit) ;
    
    for(int i = 0; i < 6; i++)
      _is >> coord[i] ;
  }
  
  /*
   * Move the ligand to the position given by the solution
   */
  void move(PTools::AttractRigidbody & ligand) {
    PTools::Coord3D center = ligand.FindCenter();
    ligand.Translate(PTools::Coord3D() - center);
    ligand.AttractEulerRotate(coord[0], coord[1], coord[2]);
    ligand.Translate(center + PTools::Coord3D(coord[3], coord[4], coord[5]));
  }
  
private:
  // fitness of the solution (energy)
  dbl energy;
  
};

#endif
