
#include "surface.h"
#include "attractrigidbody.h"

#include <cassert>

namespace PTools
{

void Surface::setUp(int nphi, int ncosth)
{
    // dimension csth(ncosth),snth(ncosth),phgh(nphi)

    const dbl pi= 3.141592654;
    //  precalculate costh,sinth and phigh for different ncosth...
    for (uint ik=0; ik < (uint) ncosth; ik++)
    {
        csth.push_back (   -1.0 + ( ( (dbl) ik+1) - 0.5  ) * (2.0 / (dbl) ncosth ) ) ;
        snth.push_back (   sqrt(1.0 -csth[ik]*csth[ik] ) );
    }

    for ( int ik=0; ik<nphi; ik++)
    {
        cos_phgh.push_back ( cos(2.0 * pi *  ((dbl)ik-1.0)/(dbl) nphi )) ;
        sin_phgh.push_back ( sin(2.0 * pi *  ((dbl)ik-1.0)/(dbl) nphi )) ;
    }

}

void Surface::surfpointParams(int max, dbl shift)
{
    m_numneh = max;
    m_sradshift = shift;
    m_init=true;
}

void Surface::readsolvparam(const std::string& file)
{
    std::string line ;
    //int count = 0;
    radi.clear();

    std::ifstream sfile(file.c_str());
    if (! sfile)
        {
            //  the file cannot be opened
            std::string msg = " Cannot Locate file ";
	    msg.append(file.c_str());
	    msg.append("\n");
            std::cerr << msg ;
            throw msg;
	}

    while (std::getline(sfile, line))
    {
        //count++ ; // count number of line (unused at the moment)
        //int size_line = line.size(); // count line length (unused at the moment)
        //dbl data1=atof(line.substr(1,6).c_str()); // read first column - atom type id (unused at the moment)
        dbl data2=atof(line.substr(7,12).c_str()); // read second column - radius (unused at the moment)
        dbl data3=atof(line.substr(13,20).c_str()); // read third column - solvation parameters (unused at the moment)
        //std::string data4=line.substr(21,size_line).c_str(); // read last column (unused at the moment)
        radi.push_back(data2);
	solv.push_back(data3);
     }
}

Rigidbody Surface::surfpoint(const Rigidbody & rigid, dbl srad, bool reduce_model)
{
    Rigidbody rigidsurf;
    int size_rigid = rigid.Size();
    std::vector<int> neigh;
    radius.clear();

    // fix neighbours parameters if not initialized
    if (!m_init)
    {
        m_numneh = 2000;
        m_sradshift = 0.0;
    }
    // read radius with attract CG file
    if (reduce_model)
	{

		AttractRigidbody rigid_tmp(rigid);
		m_atomtypenumber.resize(size_rigid);
		for (uint i=0; i< rigid_tmp.Size(); i++)
		{ m_atomtypenumber[i] = rigid_tmp.getAtomTypeNumber(i);}
		for (int i=0; i<size_rigid; i++)
		{
			Atom m_atom = rigid_tmp.CopyAtom(i);
			radius.push_back(radi[m_atomtypenumber[i]]);
		}
	}
    // read radius with pdb files
    if (!reduce_model)
	{
	std::string known_type= "CHPNO";
	for (int i=0; i<size_rigid; i++)
		{	Atom m_atom = rigid.CopyAtom(i);
			std::string m_type = m_atom.GetType();
	// why this doesn't work?
	//if ( & m_type[0] == 'C')
	//	{radi.push_back(2.0);}
	// this comparison is really ugly !! 
	// TODO: Check how to change this
			char c_m_type = m_type.at(0);
			char type_C = known_type.at(0);
			char type_H = known_type.at(1);
			char type_P = known_type.at(2);
			char type_N = known_type.at(3);
			char type_O = known_type.at(4);
			if ( c_m_type == type_C)
				{radius.push_back(2.0);}
			if ( c_m_type == type_H)
				{radius.push_back(1.0);}
			if ( c_m_type == type_P)
				{radius.push_back(2.0);}
			if ( c_m_type == type_N)
				{radius.push_back(1.6);}
			if ( c_m_type == type_O)
				{radius.push_back(1.5);}
			if (( c_m_type != type_C) && ( c_m_type != type_H) && ( c_m_type != type_P) && ( c_m_type != type_N) && ( c_m_type != type_O))
				{radius.push_back(0.0);}
		}
	}


    // generate grid points
    for (int i=0; i<size_rigid; i++)
        if ( radius[i] != 0.0 )
        {
            Coord3D coord1 = rigid.GetCoords(i);
            int numneh = 0;
            neigh.clear();
            for (int j=0; j<size_rigid; j++) // generate neighbor list
                if (i!=j)
                {
                    Coord3D coord2 = rigid.GetCoords(j);
                    dbl ccdist = Norm2(coord1 - coord2);
                    dbl rr = (radius[i]+radius[j]+2.0*srad) * (radius[i]+radius[j]+2.0*srad);
                    if (ccdist <= rr)
                    { neigh.push_back(j);
                        numneh+=1;
                        if  ( numneh > m_numneh )
                        {
                            std::string msg = " ERROR: Atom has too many neighbors \n"  ;
                            std::cout << msg;
                            throw msg;
                        }
                    }
                }
            numneh = numneh - 1;
            for (int j=0; j<m_ncosth; j++) // generate points around each atoms
            {
                dbl costh = csth[j];
                dbl sinth = snth[j];
                for (int k=0; k<m_nphi; k++)
                {
                    dbl cphigh = cos_phgh[k];
                    dbl sphigh = sin_phgh[k];
                    Coord3D coord3;
                    coord3.x = (radius[i]+srad+m_sradshift)*sinth*cphigh;
                    coord3.y = (radius[i]+srad+m_sradshift)*sinth*sphigh;
                    coord3.z = (radius[i]+srad+m_sradshift)*costh;
                    int l = 0;
                    bool coverd = false;
                    while ((!coverd) && (l <= numneh))
                    {
                        Coord3D coord5 = rigid.GetCoords(neigh[l]);
                        dbl ddd = Norm2(coord1 + coord3 - coord5);
                        if (ddd < (radius[neigh[l]] + srad+m_sradshift)*(radius[neigh[l]] + srad+m_sradshift))
                        { coverd = true; }
                        l+=1;
                    }
                    if (!coverd)
                    {
                        Atom m_atom2 = rigid.CopyAtom(i);
                        m_atom2.SetCoords(coord1 + coord3);
                        rigidsurf.AddAtom(m_atom2);
                    }
                }
            }

            // fill the top and bottom positions
            dbl costh = -1;
            while (costh<=1.0)
            {
                Coord3D coord4;
                coord4.x = 0.0;
                coord4.y = 0.0;
                coord4.z = (radius[i]+srad+m_sradshift)*costh;
                int l = 0;
                bool coverd = false;
                while ((!coverd) && (l <= numneh))
                {
                    Coord3D coord5 = rigid.GetCoords(neigh[l]);
                    dbl ddd = Norm2(coord1 + coord4 - coord5);
                    if (ddd < (radius[neigh[l]] + srad+m_sradshift)*(radius[neigh[l]] + srad+m_sradshift))
                    { coverd = true; }
                    l+=1;
                }
                if (!coverd)
                {
                    Atom m_atom2 = rigid.CopyAtom(i);
                    m_atom2.SetCoords(coord1 + coord4);
                    rigidsurf.AddAtom(m_atom2);
                }
                costh+=2;
            }
        }
    return rigidsurf;
}

Rigidbody Surface::outergrid(const Rigidbody & rigid1, const Rigidbody & rigid2, dbl srad)
{
    int size1 = rigid1.Size();
    int size2 = rigid2.Size();
    Rigidbody rigid3;
    srad=srad*srad;
    for (int i=0; i<size1; i++)
    {
        Coord3D xyz1 = rigid1.GetCoords(i);
        bool select = true;
        for (int j=0; j<size2; j++)
        {
            Coord3D xyz2 = rigid2.GetCoords(j);
            dbl dist=Norm2(xyz1-xyz2);
            if (dist < srad) { select = false; }
        }
        if (select) { rigid3.AddAtom(rigid1.CopyAtom(i)); }
    }
    return rigid3;
}

Rigidbody Surface::removeclosest(const Rigidbody & rigid, dbl srad)
{
    std::vector<bool> list,list2;
    int size=rigid.Size();
    Rigidbody rigid2;
    list.clear();
    srad=srad*srad;
    for (int i=0; i<size; i++) { list.push_back(true); }
    for (int i=0; i<size; i++)
    {
        Coord3D  xyz1 = rigid.GetCoords(i);
        for (int j=0; j<size; j++)
            if ((list[i]) && (i!=j))
            {
                Coord3D  xyz2 = rigid.GetCoords(j);
                dbl dist=Norm2(xyz1 - xyz2);
                if (dist < srad) { list[j] = false; }
            }
    }
    for (int i=0; i<size; i++)
    if (list[i]) { rigid2.AddAtom(rigid.CopyAtom(i)); }
    return rigid2;
}








std::vector<dbl> Surface::surface(const Rigidbody & rigid2,  dbl srad)
{

      AttractRigidbody rigid(rigid2);
      uint size = rigid.Size();
      int offset = 0;
      int ncosth = m_ncosth;
      int nphi = m_nphi;

      sasa.resize(size) ;
      std::vector<int> neigh (201);
      bool coverd;
      const double pi= 3.141592654;


   for (uint i=0; i<rigid.Size(); i++)
   {

     Coord3D coord1 = rigid.GetCoords(i) ;
       uint attype = rigid.getAtomTypeNumber(i);
       dbl radii=radi[attype];
       assert(sasa.size() >= i+offset);
       sasa[i+offset]=0;
       int numneh = 0;
       for (uint j=0; j<rigid.Size(); j++)
       {
        uint attypej = rigid.getAtomTypeNumber(j);
        dbl radjj=radi[attypej];
         if (j != i) 
          {
           Coord3D coord2 = rigid.GetCoords(j) ;
           dbl n2 = Norm2(coord1-coord2);
           dbl rr =(radii+radjj+2.0*srad)*(radii+radjj+2.0*srad);
             if(n2 > 0.0 && n2 <= rr)
               { 
		 assert(neigh.size() >0);
                 neigh [numneh] = j;
                 ++numneh ;
                   if (numneh > 200) 
                     {
                      std::string msg = " atom has too many neighbors \n"  ;
                      std::cout << msg ;
                        throw msg;
                      }
                }
          }
        }
        numneh = numneh - 1;
        for (int j = 0; j< ncosth;j++)
          {
          dbl costh = csth[j];
          dbl sinth = snth[j];
          for (int kk = 0; kk< nphi;kk++)
            {
            dbl cphigh = cos_phgh[kk];
            dbl sphigh = sin_phgh[kk];
            Coord3D coord3;
            coord3.x = (radii+srad)*sinth*cphigh;
            coord3.y = (radii+srad)*sinth*sphigh;
            coord3.z = (radii+srad)*costh;
            int ll = 0;
            coverd = false ;
l700:          if( (!coverd) && (ll<=numneh) )
              {
                assert(neigh.size()>0);
                int nu=neigh[ll];
		uint attypenu = rigid.getAtomTypeNumber(nu);
                Coord3D coord2 = rigid.GetCoords(nu) ;
                dbl n3 = Norm2(coord1 + coord3 -coord2); 
                if(n3 < (radi[attypenu] + srad)*(radi[attypenu] + srad))
                  {
                   coverd = true;
                  }
                ll = ll + 1;
                goto l700;
               } 
               if (!coverd) 
               {
                sasa[i+offset]=sasa[i+offset]+1.0;
               }
             }
           }
           sasa[i+offset] = (radii+srad)*(radii+srad)*(2.0/(double) ncosth)* (2.0*pi/nphi)*sasa[i+offset];
   }

  return sasa;

}


std::vector<dbl> Surface::solvation(const Rigidbody & rigid2,  std::vector<dbl> surf)
{
        AttractRigidbody rigid(rigid2);
	uint size_rigid = rigid.Size();
	uint size_surf = surf.size();
	sasa_solv.resize(size_surf);
	if (size_rigid!=size_surf)
		{
		std::string msg = " surface size is not equal to protein size! \n"  ;
		std::cout << msg ;
		}
	if (size_rigid==size_surf)
		{
		for (uint i=0; i<rigid.Size(); i++)
			{
			uint attype = rigid.getAtomTypeNumber(i);
			sasa_solv[i]=sasa[i]*solv[attype];
			}
		}
	return sasa_solv;

}













}//namespace PTools
